@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2020 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2021 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -15,7 +15,7 @@ from time import time
 from random import shuffle
 
 from lollypop.sqlcursor import SqlCursor
-from lollypop.define import App, Type, OrderBy, StorageType
+from lollypop.define import App, Type, OrderBy, StorageType, LovedFlags
 from lollypop.logger import Logger
 from lollypop.utils import remove_static, make_subrequest
 
@@ -49,7 +49,6 @@ class AlbumsDatabase:
             @param mtime as int
             @param storage_type as int
             @return inserted rowid as int
-            @warning: commit needed
         """
         with SqlCursor(self.__db, True) as sql:
             result = sql.execute("INSERT INTO albums\
@@ -72,7 +71,6 @@ class AlbumsDatabase:
             Add artist to track
             @param album_id as int
             @param artist_id as int
-            @warning: commit needed
         """
         with SqlCursor(self.__db, True) as sql:
             artist_ids = self.get_artist_ids(album_id)
@@ -86,7 +84,6 @@ class AlbumsDatabase:
             Add genre to album
             @param album_id as int
             @param genre_id as int
-            @warning: commit needed
         """
         with SqlCursor(self.__db, True) as sql:
             genres = self.get_genre_ids(album_id)
@@ -101,7 +98,6 @@ class AlbumsDatabase:
             Set artist id
             @param album_id as int
             @param artist_ids as [int]
-            @warning: commit needed
         """
         with SqlCursor(self.__db, True) as sql:
             sql.execute("DELETE FROM album_artists\
@@ -116,7 +112,6 @@ class AlbumsDatabase:
             Set album synced
             @param album_id as int
             @param synced as int
-            @warning: commit needed
         """
         with SqlCursor(self.__db, True) as sql:
             sql.execute("UPDATE albums SET synced=? WHERE rowid=?",
@@ -147,7 +142,6 @@ class AlbumsDatabase:
             Set album loved
             @param album_id as int
             @param loved as int
-            @warning: commit needed
         """
         with SqlCursor(self.__db, True) as sql:
             sql.execute("UPDATE albums SET loved=? WHERE rowid=?",
@@ -168,7 +162,6 @@ class AlbumsDatabase:
             Set year
             @param album_id as int
             @param year as int
-            @warning: commit needed
         """
         with SqlCursor(self.__db, True) as sql:
             sql.execute("UPDATE albums SET year=? WHERE rowid=?",
@@ -179,7 +172,6 @@ class AlbumsDatabase:
             Set timestamp
             @param album_id as int
             @param timestamp as int
-            @warning: commit needed
         """
         with SqlCursor(self.__db, True) as sql:
             sql.execute("UPDATE albums SET timestamp=? WHERE rowid=?",
@@ -190,7 +182,6 @@ class AlbumsDatabase:
             Set album uri for album id
             @param album_id as int
             @param uri as string
-            @warning: commit needed
         """
         with SqlCursor(self.__db, True) as sql:
             sql.execute("UPDATE albums SET uri=? WHERE rowid=?",
@@ -702,13 +693,16 @@ class AlbumsDatabase:
             @return [int]
         """
         with SqlCursor(self.__db) as sql:
+            filters = (storage_type,)
             request = "SELECT DISTINCT albums.rowid\
                        FROM albums\
                        WHERE rate>=4 AND storage_type & ?"
             if not skipped:
-                request += " AND loved != -1 "
+                request += " AND not loved & ?"
+                filters += (LovedFlags.SKIPPED,)
             request += "ORDER BY popularity DESC LIMIT ?"
-            result = sql.execute(request, (storage_type, limit))
+            filters += (limit,)
+            result = sql.execute(request, filters)
             return list(itertools.chain(*result))
 
     def get_populars(self, storage_type, skipped, limit):
@@ -720,13 +714,15 @@ class AlbumsDatabase:
             @return [int]
         """
         with SqlCursor(self.__db) as sql:
+            filters = (storage_type,)
             request = "SELECT DISTINCT albums.rowid FROM albums\
                        WHERE popularity!=0 AND storage_type & ?"
             if not skipped:
-                request += " AND loved != -1 "
+                request += " AND not loved & ?"
+                filters += (LovedFlags.SKIPPED,)
             request += "ORDER BY popularity DESC LIMIT ?"
-            result = sql.execute(request,
-                                 (storage_type, limit))
+            filters += (limit,)
+            result = sql.execute(request, filters)
             return list(itertools.chain(*result))
 
     def get_populars_at_the_moment(self, storage_type, skipped, limit):
@@ -738,15 +734,18 @@ class AlbumsDatabase:
             @return [int]
         """
         with SqlCursor(self.__db) as sql:
+            filters = (storage_type,)
             request = "SELECT DISTINCT albums.rowid\
                        FROM albums, albums_timed_popularity\
                        WHERE albums.storage_type & ? AND\
                              albums.rowid = albums_timed_popularity.album_id"
             if not skipped:
-                request += " AND loved != -1 "
+                request += " AND not loved & ?"
+                filters += (LovedFlags.SKIPPED,)
             request += "ORDER BY albums_timed_popularity.popularity DESC\
                         LIMIT ?"
-            result = sql.execute(request, (storage_type, limit))
+            filters += (limit,)
+            result = sql.execute(request, filters)
             album_ids = list(itertools.chain(*result))
             if album_ids:
                 return album_ids
@@ -761,9 +760,9 @@ class AlbumsDatabase:
         with SqlCursor(self.__db) as sql:
             request = "SELECT albums.rowid\
                        FROM albums\
-                       WHERE loved=1 AND\
+                       WHERE loved & ? AND\
                        storage_type & ? ORDER BY popularity DESC"
-            result = sql.execute(request, (storage_type,))
+            result = sql.execute(request, (LovedFlags.LOVED, storage_type,))
             return list(itertools.chain(*result))
 
     def get_recents(self, storage_type, skipped, limit):
@@ -775,12 +774,15 @@ class AlbumsDatabase:
             @return [int]
         """
         with SqlCursor(self.__db) as sql:
+            filters = (storage_type,)
             request = "SELECT DISTINCT albums.rowid FROM albums\
                        WHERE albums.storage_type & ?"
             if not skipped:
-                request += " AND loved != -1 "
+                request += " AND not loved & ?"
+                filters += (LovedFlags.SKIPPED,)
             request += "ORDER BY mtime DESC LIMIT ?"
-            result = sql.execute(request, (storage_type, limit))
+            filters += (limit,)
+            result = sql.execute(request, filters)
             return list(itertools.chain(*result))
 
     def get_randoms_by_albums(self, storage_type, genre_id, skipped, limit):
@@ -794,23 +796,27 @@ class AlbumsDatabase:
         """
         with SqlCursor(self.__db) as sql:
             if genre_id is not None:
-                filter = (storage_type, genre_id, limit)
+                filters = (storage_type, genre_id)
                 request = "SELECT DISTINCT albums.rowid\
                            FROM albums, album_genres\
                            WHERE albums.storage_type & ? AND\
                                  album_genres.album_id = albums.rowid AND\
                                  album_genres.genre_id = ?"
                 if not skipped:
-                    request += " AND loved != -1 "
+                    request += " AND not loved & ?"
+                    filters += (LovedFlags.SKIPPED,)
                 request += "ORDER BY random() LIMIT ?"
+                filters += (limit,)
             else:
-                filter = (storage_type, limit)
+                filters = (storage_type,)
                 request = "SELECT DISTINCT rowid FROM albums\
                            WHERE storage_type & ?"
                 if not skipped:
-                    request += " AND loved != -1 "
+                    request += " AND not loved & ?"
+                    filters += (LovedFlags.SKIPPED,)
                 request += "ORDER BY random() LIMIT ?"
-            result = sql.execute(request, filter)
+                filters += (limit,)
+            result = sql.execute(request, filters)
             albums = list(itertools.chain(*result))
             return albums
 
@@ -825,7 +831,7 @@ class AlbumsDatabase:
         """
         with SqlCursor(self.__db) as sql:
             if genre_id is not None:
-                filter = (storage_type, genre_id, limit * 2, limit)
+                filters = (storage_type, genre_id)
                 request = "SELECT rowid, artist_id FROM (\
                                SELECT albums.rowid, album_artists.artist_id\
                                FROM albums, album_genres, album_artists\
@@ -834,22 +840,26 @@ class AlbumsDatabase:
                                      album_genres.album_id = albums.rowid AND\
                                      album_genres.genre_id = ?"
                 if not skipped:
-                    request += " AND loved != -1 "
+                    request += " AND not loved & ?"
+                    filters += (LovedFlags.SKIPPED,)
+                filters += (limit * 2, limit)
                 request += "ORDER BY random() LIMIT ?)\
                             GROUP BY artist_id ORDER BY random() LIMIT ?"
             else:
-                filter = (storage_type, limit * 2, limit)
+                filters = (storage_type,)
                 request = "SELECT rowid, artist_id FROM (\
                                SELECT albums.rowid, album_artists.artist_id\
                                FROM albums, album_artists\
                                WHERE albums.rowid = album_artists.album_id AND\
                                      albums.storage_type & ?"
                 if not skipped:
-                    request += " AND loved != -1 "
+                    request += " AND not loved & ?"
+                    filters += (LovedFlags.SKIPPED,)
+                filters += (limit * 2, limit)
                 request += "ORDER BY random() LIMIT ?)\
                             GROUP BY artist_id ORDER BY random() LIMIT ?"
             album_ids = []
-            for (album_id, artist_id) in sql.execute(request, filter):
+            for (album_id, artist_id) in sql.execute(request, filters):
                 album_ids.append(album_id)
             return album_ids
 
@@ -954,14 +964,15 @@ class AlbumsDatabase:
                                            "OR",
                                            len(artist_ids))
             if not skipped:
-                request += " AND tracks.loved != -1"
+                request += " AND not tracks.loved & ?"
+                filters += (LovedFlags.SKIPPED,)
             request += " ORDER BY discnumber, tracknumber, tracks.name"
             result = sql.execute(request, filters)
             return list(itertools.chain(*result))
 
     def get_tracks_count(self, album_id, genre_ids, artist_ids):
         """
-            Get tracks ids for album
+            Get tracks count for album
             @param album_id as int
             @param genre_ids as [int]
             @param artist_ids as [int]
@@ -1057,13 +1068,15 @@ class AlbumsDatabase:
             result = []
             # Get albums for all artists
             if not artist_ids and not genre_ids:
+                filters = (storage_type,)
                 request = "SELECT DISTINCT albums.rowid\
                            FROM albums, album_artists, artists\
                            WHERE albums.rowid = album_artists.album_id AND\
                            albums.storage_type & ? AND\
                            artists.rowid = album_artists.artist_id"
                 if not skipped:
-                    request += " AND albums.loved != -1"
+                    request += " AND not albums.loved & ?"
+                    filters += (LovedFlags.SKIPPED,)
                 request += order
                 result = sql.execute(request, (storage_type,))
             # Get albums for genres
@@ -1080,7 +1093,8 @@ class AlbumsDatabase:
                                            "OR",
                                            len(genre_ids))
                 if not skipped:
-                    request += " AND albums.loved != -1"
+                    request += " AND not albums.loved & ?"
+                    filters += (LovedFlags.SKIPPED,)
                 request += order
                 result = sql.execute(request, filters)
             # Get albums for artist
@@ -1096,7 +1110,8 @@ class AlbumsDatabase:
                                            "OR",
                                            len(artist_ids))
                 if not skipped:
-                    request += " AND albums.loved != -1"
+                    request += " AND not albums.loved & ?"
+                    filters += (LovedFlags.SKIPPED,)
                 request += order
                 result = sql.execute(request, filters)
             # Get albums for artist id and genre id
@@ -1118,7 +1133,8 @@ class AlbumsDatabase:
                                            "OR",
                                            len(genre_ids))
                 if not skipped:
-                    request += " AND albums.loved != -1"
+                    request += " AND not albums.loved & ?"
+                    filters += (LovedFlags.SKIPPED,)
                 request += order
                 result = sql.execute(request, filters)
             return list(itertools.chain(*result))
@@ -1144,7 +1160,8 @@ class AlbumsDatabase:
                            AND album_artists.artist_id=?\
                            AND album_artists.album_id=albums.rowid"
                 if not skipped:
-                    request += " AND albums.loved != -1"
+                    request += " AND not albums.loved & ?"
+                    filters += (LovedFlags.SKIPPED,)
                 request += order
                 result = sql.execute(request, filters)
             # Get compilation for genre id
@@ -1155,33 +1172,46 @@ class AlbumsDatabase:
                            FROM albums, album_genres, album_artists\
                            WHERE album_genres.album_id=albums.rowid\
                            AND albums.storage_type & ?\
-                           AND albums.loved != -1\
                            AND album_artists.album_id=albums.rowid\
                            AND album_artists.artist_id=? AND"
                 request += make_subrequest("album_genres.genre_id=?",
                                            "OR",
                                            len(genre_ids))
                 if not skipped:
-                    request += " AND albums.loved != -1"
+                    request += " AND not albums.loved & ?"
+                    filters += (LovedFlags.SKIPPED,)
                 request += order
                 result = sql.execute(request, filters)
             return list(itertools.chain(*result))
 
-    def get_duration(self, album_id, genre_ids, artist_ids):
+    def get_duration(self, album_id, genre_ids, artist_ids, disc_number):
         """
             Album duration in seconds
             @param album_id as int
             @param genre_ids as [int]
-            @return album duration as int
+            @param artist_ids as [int]
+            @param disc_number as int/None
+            @return int
         """
         genre_ids = remove_static(genre_ids)
         artist_ids = remove_static(artist_ids)
         with SqlCursor(self.__db) as sql:
-            if not genre_ids and not artist_ids:
+            if genre_ids and artist_ids:
                 filters = (album_id,)
+                filters += tuple(genre_ids)
+                filters += tuple(artist_ids)
                 request = "SELECT SUM(duration)\
-                           FROM tracks\
-                           WHERE tracks.album_id=?"
+                           FROM tracks, track_genres, track_artists\
+                           WHERE tracks.album_id=?\
+                           AND track_genres.track_id = tracks.rowid\
+                           AND track_artists.track_id = tracks.rowid AND"
+                request += make_subrequest("track_genres.genre_id=?",
+                                           "OR",
+                                           len(genre_ids))
+                request += " AND "
+                request += make_subrequest("track_artists.artist_id=?",
+                                           "OR",
+                                           len(artist_ids))
             elif artist_ids:
                 filters = (album_id,)
                 filters += tuple(artist_ids)
@@ -1204,19 +1234,12 @@ class AlbumsDatabase:
                                            len(genre_ids))
             else:
                 filters = (album_id,)
-                filters += tuple(genre_ids)
-                filters += tuple(artist_ids)
                 request = "SELECT SUM(duration)\
-                           FROM tracks, track_genres, track_artists\
-                           WHERE tracks.album_id=?\
-                           AND track_genres.track_id = tracks.rowid\
-                           AND track_artists.track_id = tracks.rowid AND"
-                request += make_subrequest("track_genres.genre_id=?",
-                                           "OR",
-                                           len(genre_ids))
-                request += make_subrequest("track_artists.artist_id=?",
-                                           "OR",
-                                           len(artist_ids))
+                           FROM tracks\
+                           WHERE tracks.album_id=?"
+            if disc_number is not None:
+                filters += (disc_number,)
+                request += " AND discnumber=?"
             result = sql.execute(request, filters)
             v = result.fetchone()
             if v and v[0] is not None:
@@ -1245,103 +1268,16 @@ class AlbumsDatabase:
             @return album ids as [int]
         """
         with SqlCursor(self.__db) as sql:
+            filters = ()
             request = "SELECT album_id FROM tracks, albums\
                        WHERE albums.storage_type & ? AND albums.rowid=album_id"
             if not skipped:
-                request += " AND albums.loved != -1"
+                request += " AND not albums.loved & ?"
+                filters += (LovedFlags.SKIPPED,)
             request += " GROUP BY album_id\
                         ORDER BY SUM(ltime)/COUNT(ltime), random() LIMIT ?"
-            result = sql.execute(request, (storage_type, limit))
-            return list(itertools.chain(*result))
-
-    def get_years(self, storage_type):
-        """
-            Return all albums years and if unknown album exists
-            @param storage_type as StorageType
-            @return ([int], bool)
-        """
-        with SqlCursor(self.__db) as sql:
-            result = sql.execute("SELECT albums.year\
-                                  FROM albums\
-                                  WHERE storage_type & ?",
-                                 (storage_type,))
-            years = []
-            unknown = False
-            for year in list(itertools.chain(*result)):
-                if year is None:
-                    unknown = True
-                elif year not in years:
-                    years.append(year)
-            return (years, unknown)
-
-    def get_ids_for_year(self, year, storage_type, skipped, limit=-1):
-        """
-            Return albums for year
-            @param year as int
-            @param storage_type as StorageType
-            @param skipped as bool
-            @param limit as int
-            @return album ids as [int]
-        """
-        with SqlCursor(self.__db) as sql:
-            order = " ORDER BY artists.sortname\
-                     COLLATE NOCASE COLLATE LOCALIZED,\
-                     albums.timestamp,\
-                     albums.name\
-                     COLLATE NOCASE COLLATE LOCALIZED LIMIT ?"
-            if year == Type.NONE:
-                request = "SELECT DISTINCT albums.rowid\
-                           FROM albums, album_artists, artists\
-                           WHERE albums.rowid=album_artists.album_id AND\
-                           artists.rowid=album_artists.artist_id AND\
-                           albums.year is null AND albums.storage_type & ?"
-                filter = (storage_type, limit)
-            else:
-                request = "SELECT DISTINCT albums.rowid\
-                           FROM albums, album_artists, artists\
-                           WHERE albums.rowid=album_artists.album_id AND\
-                           artists.rowid=album_artists.artist_id AND\
-                           albums.year=? AND albums.storage_type & ?"
-                filter = (year, storage_type, limit)
-            if not skipped:
-                request += " AND albums.loved != -1"
-            request += order
-            result = sql.execute(request, filter)
-            return list(itertools.chain(*result))
-
-    def get_compilation_ids_for_year(self, year, storage_type,
-                                     skipped, limit=-1):
-        """
-            Return compilations for year
-            @param year as int
-            @param storage_type as StorageType
-            @param skipped as bool
-            @param limit as int
-            @return album ids as [int]
-        """
-        with SqlCursor(self.__db) as sql:
-            order = " ORDER BY albums.timestamp, albums.name\
-                     COLLATE NOCASE COLLATE LOCALIZED LIMIT ?"
-            if year == Type.NONE:
-                request = "SELECT DISTINCT albums.rowid\
-                           FROM albums, album_artists\
-                           WHERE album_artists.artist_id=?\
-                           AND album_artists.album_id=albums.rowid\
-                           AND albums.storage_type & ?\
-                           AND albums.year is null"
-                filter = (Type.COMPILATIONS, storage_type, limit)
-            else:
-                request = "SELECT DISTINCT albums.rowid\
-                           FROM albums, album_artists\
-                           WHERE album_artists.artist_id=?\
-                           AND album_artists.album_id=albums.rowid\
-                           AND albums.storage_type & ?\
-                           AND albums.year=?"
-                filter = (Type.COMPILATIONS, storage_type, year, limit)
-            if not skipped:
-                request += " AND albums.loved != -1"
-            request += order
-            result = sql.execute(request, filter)
+            filters += (storage_type, limit)
+            result = sql.execute(request, filters)
             return list(itertools.chain(*result))
 
     def search(self, searched, storage_type):
