@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2020 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2021 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # Copyright (c) 2015 Jean-Philippe Braun <eon@patapon.info>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 
 from hashlib import md5
 
-from lollypop.define import App, StorageType, ScanUpdate
+from lollypop.define import App, StorageType, ScanUpdate, Type
 from lollypop.objects_track import Track
 from lollypop.objects import Base
 from lollypop.utils import emit_signal
@@ -96,12 +96,12 @@ class Disc:
         if not self.__tracks and self.album.id is not None:
             self.__tracks = [Track(track_id, self.album)
                              for track_id in self.db.get_disc_track_ids(
-                self.album.id,
-                self.album.genre_ids,
-                self.album.artist_ids,
-                self.number,
-                self.__storage_type,
-                self.__skipped)]
+                                    self.album.id,
+                                    self.album.genre_ids,
+                                    self.album.artist_ids,
+                                    self.number,
+                                    self.__storage_type,
+                                    self.__skipped)]
         return self.__tracks
 
 
@@ -109,8 +109,7 @@ class Album(Base):
     """
         Represent an album
     """
-    DEFAULTS = {"name": "",
-                "artists": [],
+    DEFAULTS = {"artists": [],
                 "artist_ids": [],
                 "year": None,
                 "timestamp": 0,
@@ -136,10 +135,12 @@ class Album(Base):
         Base.__init__(self, App().albums)
         self.id = album_id
         self.genre_ids = genre_ids
-        self._tracks = []
-        self._discs = []
+        self.__tracks = []
+        self.__discs = []
+        self.__name = None
         self.__skipped = skipped
-        self.__one_disc = None
+        self.__disc_number = None
+        self.__original_year = Type.NONE
         self.__tracks_storage_type = self.storage_type
         # Use artist ids from db else
         if artist_ids:
@@ -169,7 +170,15 @@ class Album(Base):
             Set album discs
             @param discs as [Disc]
         """
-        self._discs = discs
+        self.__discs = discs
+
+    def set_disc_number(self, disc_number):
+        """
+            Set album disc
+            @param disc_number as int
+        """
+        self.__original_year = Type.NONE
+        self.__disc_number = disc_number
 
     def set_tracks(self, tracks, clone=True):
         """
@@ -179,38 +188,41 @@ class Album(Base):
             @param clone as bool
         """
         if clone:
-            self._tracks = []
+            self.__tracks = []
             for track in tracks:
                 new_track = Track(track.id, self)
-                self._tracks.append(new_track)
-        else:
-            # Create a new album for current tracks
+                self.__tracks.append(new_track)
+        # Album tracks already belong to self
+        # Detach those tracks
+        elif self.__tracks:
             new_album = Album(self.id, self.genre_ids, self.artist_ids)
-            new_tracks = []
-            for track in self._tracks:
+            new__tracks = []
+            for track in self.__tracks:
                 if track not in tracks:
                     track.set_album(new_album)
-                    new_tracks.append(track)
-            new_album._tracks = new_tracks
-            self._tracks = tracks
+                    new__tracks.append(track)
+            new_album.__tracks = new__tracks
+            self.__tracks = tracks
+        else:
+            self.__tracks = tracks
 
     def append_track(self, track, clone=True):
         """
-            Append track to album, do not disable clone if you know self is
-            already used
+            Append track to album.
+            Clone: always do this if track is used in UI/Player
             @param track as Track
             @param clone as bool
         """
         if clone:
-            self._tracks.append(Track(track.id, self))
+            self.__tracks.append(Track(track.id, self))
         else:
-            self._tracks.append(track)
+            self.__tracks.append(track)
             track.set_album(self)
 
     def append_tracks(self, tracks, clone=True):
         """
-            Append tracks to album, do not disable clone if you know self is
-            already used
+            Append tracks to album
+            Clone: always do this if track is used in UI/Player
             @param tracks as [Track]
             @param clone as bool
         """
@@ -219,30 +231,34 @@ class Album(Base):
 
     def remove_track(self, track):
         """
-            Remove track from album
+            Remove track from album, album id is None if empty
             @param track as Track
-            @return True if album empty
         """
         for _track in self.tracks:
             if track.id == _track.id:
-                self._tracks.remove(_track)
-        return len(self._tracks) == 0
+                self.__tracks.remove(_track)
+        empty = len(self.__tracks) == 0
+        if empty:
+            # We don't the album to load tracks anymore
+            self.id = None
 
     def reset_tracks(self):
         """
-            Reset album tracks
+            Reset album tracks, useful for tracks loaded async
         """
-        self._tracks = []
-        self._discs = []
+        self.__tracks = []
+        self.__discs = []
+        self.reset("artists")
+        self.reset("artist_ids")
         self.reset("lp_album_id")
 
-    def disc_names(self, disc):
+    def disc_names(self, disc_number):
         """
             Disc names
-            @param disc as int
+            @param disc_number as int
             @return disc names as [str]
         """
-        return self.db.get_disc_names(self.id, disc)
+        return self.db.get_disc_names(self.id, disc_number)
 
     def set_loved(self, loved):
         """
@@ -333,7 +349,9 @@ class Album(Base):
             @param skipped as bool
             @return album
         """
-        return Album(self.id, self.genre_ids, self.artist_ids, skipped)
+        album = Album(self.id, self.genre_ids, self.artist_ids, skipped)
+        album.set_tracks(self.tracks)
+        return album
 
     def set_storage_type(self, storage_type):
         """
@@ -348,6 +366,28 @@ class Album(Base):
         """
         self.__skipped = True
 
+    def merge_discs(self):
+        """
+            Merge album discs
+            @return Disc
+        """
+        self.__original_year = None
+        tracks = self.tracks
+        disc = Disc(self, 0, self.__tracks_storage_type, self.__skipped)
+        disc.set_tracks(tracks)
+        self.__discs = [disc]
+
+    @property
+    def original_year(self):
+        """
+            Get disc original year
+            @return int/None
+        """
+        if self.__original_year == Type.NONE:
+            self.__original_year = App().tracks.get_year_for_album(
+                self.id, self.__disc_number)
+        return self.__original_year
+
     @property
     def collection_item(self):
         """
@@ -360,6 +400,24 @@ class Album(Base):
                               lp_album_id=self.lp_album_id)
         return item
 
+    @property
+    def name(self):
+        """
+            Get album name
+            @return str
+        """
+        if self.__name is not None:
+            return self.__name
+        if self.__disc_number is None:
+            self.__name = self.db.get_name(self.id)
+        else:
+            disc_names = self.disc_names(self.__disc_number)
+            if disc_names:
+                self.__name = ", ".join(disc_names)
+            else:
+                self.__name = self.db.get_name(self.id)
+        return self.__name
+
     @property
     def is_web(self):
         """
@@ -375,10 +433,10 @@ class Album(Base):
             Get tracks count
             @return int
         """
-        if self._tracks:
-            return len(self._tracks)
+        if self.__tracks:
+            return len(self.__tracks)
         else:
-            return self.db.get_tracks_count(
+            return self.db.get__tracks_count(
                 self.id,
                 self.genre_ids,
                 self.artist_ids)
@@ -405,23 +463,17 @@ class Album(Base):
             Get album tracks
             @return [Track]
         """
-        if not self._tracks and self.id is not None:
-            for disc in self.discs:
-                self._tracks += disc.tracks
-        return self._tracks
-
-    @property
-    def one_disc(self):
-        """
-            Get album as one disc
-            @return Disc
-        """
-        if self.__one_disc is None:
-            tracks = self.tracks
-            self.__one_disc = Disc(self, 0, self.__tracks_storage_type,
-                                   self.__skipped)
-            self.__one_disc.set_tracks(tracks)
-        return self.__one_disc
+        if self.id is None:
+            return []
+        if self.__tracks:
+            return self.__tracks
+        tracks = []
+        for disc in self.discs:
+            tracks += disc.tracks
+        # Already cached by another thread
+        if not self.__tracks:
+            self.__tracks = tracks
+        return tracks
 
     @property
     def discs(self):
@@ -429,15 +481,23 @@ class Album(Base):
             Get albums discs
             @return [Disc]
         """
-        if not self._discs:
+        if self.__discs:
+            return self.__discs
+        discs = []
+        if self.__disc_number is None:
             disc_numbers = self.db.get_discs(self.id)
-            for disc_number in disc_numbers:
-                disc = Disc(self, disc_number,
-                            self.__tracks_storage_type,
-                            self.__skipped)
-                if disc.tracks:
-                    self._discs.append(disc)
-        return self._discs
+        else:
+            disc_numbers = [self.__disc_number]
+        for disc_number in disc_numbers:
+            disc = Disc(self, disc_number,
+                        self.__tracks_storage_type,
+                        self.__skipped)
+            if disc.tracks:
+                discs.append(disc)
+        # Already cached by another thread
+        if not self.__discs:
+            self.__discs = discs
+        return self.__discs
 
     @property
     def duration(self):
@@ -445,25 +505,28 @@ class Album(Base):
             Get album duration and handle caching
             @return int
         """
-        if self._tracks:
+        if self.__tracks:
             track_ids = [track.lp_track_id for track in self.tracks]
             track_str = "%s" % sorted(track_ids)
             track_hash = md5(track_str.encode("utf-8")).hexdigest()
-            album_hash = "%s-%s" % (self.lp_album_id, track_hash)
+            album_hash = "%s-%s-%s" % (
+                self.lp_album_id, track_hash, self.__disc_number)
         else:
-            album_hash = "%s-%s-%s" % (self.lp_album_id,
-                                       self.genre_ids,
-                                       self.artist_ids)
+            album_hash = "%s-%s-%s-%s" % (self.lp_album_id,
+                                          self.genre_ids,
+                                          self.artist_ids,
+                                          self.__disc_number)
         duration = App().cache.get_duration(album_hash)
         if duration is None:
-            if self._tracks:
+            if self.__tracks:
                 duration = 0
-                for track in self._tracks:
+                for track in self.__tracks:
                     duration += track.duration
             else:
                 duration = self.db.get_duration(self.id,
                                                 self.genre_ids,
-                                                self.artist_ids)
+                                                self.artist_ids,
+                                                self.__disc_number)
             App().cache.set_duration(self.id, album_hash, duration)
         return duration
 
