@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2020 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2021 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -348,7 +348,12 @@ class TagReader:
             (exists, m) = sample.get_buffer().map(Gst.MapFlags.READ)
             if not exists:
                 continue
-            frame = FrameTextTag(m.data)
+            # Gstreamer 1.18 API breakage
+            try:
+                bytes = m.data.tobytes()
+            except:
+                bytes = m.data
+            frame = FrameTextTag(bytes)
             if frame.key == "TCMP":
                 string = frame.string
                 if not string:
@@ -442,7 +447,12 @@ class TagReader:
                     (exists, m) = sample.get_buffer().map(Gst.MapFlags.READ)
                     if not exists:
                         continue
-                    frame = FrameTextTag(m.data)
+                    # Gstreamer 1.18 API breakage
+                    try:
+                        bytes = m.data.tobytes()
+                    except:
+                        bytes = m.data
+                    frame = FrameTextTag(bytes)
                     if frame.key == "TDOR":
                         if not frame.string:
                             Logger.debug(tags.to_string())
@@ -509,62 +519,22 @@ class TagReader:
                 (exists, m) = sample.get_buffer().map(Gst.MapFlags.READ)
                 if not exists:
                     continue
-                if m.data[0:4] == b"POPM":
-                    # Get tag
-                    """
-                        Location of rating for common media players:
-                        - Plain POPM is located in m.data[11]
-                        - Media Monkey is located in m.data[19] and
-                          we are searching m.data[10:40] for "no@email"
-                        - WinAMP is located in m.data[28] and we are searching
-                          m.data[10:40] for "rating@winamp.com"
-                        - Windows Media Player is located in m.data[40] and
-                          we are searching m.data[10:40] for
-                          "Windows Media Player 9 Series"
-                        - MusicBee is located in m.data[19] and
-                          we are searching m.data[10:40] for
-                          "MusicBee"
-                    """
-                    # Gstreamer 1.18 API breakage
-                    try:
-                        bytes = m.data.tobytes()
-                    except:
-                        bytes = m.data
-                    if bytes.find(b"Windows Media Player 9 Series") >= 0:
-                        location = 40
-                        Logger.debug(
-                            "Rating type: Windows Media Player Location:"
-                            " %s Rating: %s", location, bytes[location])
-                    elif bytes.find(b"rating@winamp.com") >= 0:
-                        location = 28
-                        Logger.debug(
-                            "Rating type: WinAMP Location: %s Rating: %s",
-                            location, bytes[location])
-                    elif bytes.find(b"no@email") >= 0:
-                        location = 19
-                        Logger.debug(
-                            "Rating type: MediaMonkey Location: %s Rating: %s",
-                            location, bytes[location])
-                    elif bytes.find(b"MusicBee") >= 0:
-                        location = 19
-                        Logger.debug(
-                            "Rating type: MusicBee Location: %s Rating: %s",
-                            location, bytes[location])
-                    else:
-                        location = 11
-                        Logger.debug(
-                            "Rating type: none Location: %s Rating: %s",
-                            location, bytes[location])
-                    popm = bytes[location]
+                # Gstreamer 1.18 API breakage
+                try:
+                    bytes = m.data.tobytes()
+                except:
+                    bytes = m.data
+                if bytes[0:4] == b"POPM":
+                    popm = bytes.split(b"\x00")[6][0]
                     if popm == 0:
                         value = 0
-                    elif popm == 1:
+                    elif popm >= 1 and popm < 64:
                         value = 1
-                    elif popm == 64:
+                    elif popm >= 64 and popm < 128:
                         value = 2
-                    elif popm == 128:
+                    elif popm >= 128 and popm < 196:
                         value = 3
-                    elif popm == 196:
+                    elif popm >= 196 and popm < 255:
                         value = 4
                     elif popm == 255:
                         value = 5
@@ -611,7 +581,12 @@ class TagReader:
                     (exists, m) = sample.get_buffer().map(Gst.MapFlags.READ)
                     if not exists:
                         continue
-                    string = decode_lyrics(m.data)
+                    # Gstreamer 1.18 API breakage
+                    try:
+                        bytes = m.data.tobytes()
+                    except:
+                        bytes = m.data
+                    string = decode_lyrics(bytes)
                     if string is not None:
                         return string
             except Exception as e:
@@ -671,10 +646,15 @@ class TagReader:
                     (exists, m) = sample.get_buffer().map(Gst.MapFlags.READ)
                     if not exists:
                         continue
-                    prefix = (m.data[0:4])
+                    # Gstreamer 1.18 API breakage
+                    try:
+                        bytes = m.data.tobytes()
+                    except:
+                        bytes = m.data
+                    prefix = (bytes[0:4])
                     if prefix not in [b"SYLT"]:
                         continue
-                    frame = m.data[10:]
+                    frame = bytes[10:]
                     encoding = frame[0:1]
                     string = decode_lyrics(frame.split(b"\n"), encoding)
                     if string is not None:
