@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2020 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2021 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -15,10 +15,10 @@ from gi.repository import GLib, Gtk, Pango
 from gettext import gettext as _
 
 from lollypop.widgets_row_track import TrackRow
+from lollypop.widgets_label import LabelWidget
 from lollypop.objects_album import Album
-from lollypop.logger import Logger
-from lollypop.utils import set_cursor_type, emit_signal
-from lollypop.define import App, Type, ViewType
+from lollypop.utils import emit_signal
+from lollypop.define import App, ViewType
 from lollypop.view_tracks import TracksView
 
 
@@ -35,8 +35,9 @@ class AlbumTracksView(TracksView):
         """
         TracksView.__init__(self, view_type)
         self.__album = album
-        self.__discs = []
         self.__discs_to_load = []
+        self.__disc_labels = {}
+        self.__show_disc_header = False
         self.__populated = False
         self.__show_tag_tracknumber = App().settings.get_value(
             "show-tag-tracknumber")
@@ -59,6 +60,7 @@ class AlbumTracksView(TracksView):
                               tracks[:mid_tracks]))
                 items.append((self._tracks_widget_right[disc_number],
                               tracks[mid_tracks:]))
+            self.__set_disc_header(disc_number)
             self.__load_disc(items, disc_number)
         else:
             self.__populated = True
@@ -108,41 +110,6 @@ class AlbumTracksView(TracksView):
             if child.track == track:
                 child.destroy()
 
-    def set_playing_indicator(self):
-        """
-            Set playing indicator
-        """
-        try:
-            for disc in self.__discs:
-                self._tracks_widget_left[disc.number].update_playing(
-                    App().player.current_track.id)
-                self._tracks_widget_right[disc.number].update_playing(
-                    App().player.current_track.id)
-        except Exception as e:
-            Logger.error("TrackView::set_playing_indicator(): %s" % e)
-
-    def update_duration(self, track_id):
-        """
-            Update track duration
-            @param track_id as int
-        """
-        try:
-            for disc in self.__discs:
-                number = disc.number
-                self._tracks_widget_left[number].update_duration(track_id)
-                self._tracks_widget_right[number].update_duration(track_id)
-        except Exception as e:
-            Logger.error("TrackView::update_duration(): %s" % e)
-
-    def set_view_type(self, view_type):
-        """
-            Update view type
-            @param view_type as ViewType
-        """
-        self._view_type = view_type
-        for child in self.children:
-            child.set_view_type(view_type)
-
     @property
     def children(self):
         """
@@ -150,7 +117,7 @@ class AlbumTracksView(TracksView):
             @return [Gtk.ListBoxRow]
         """
         rows = []
-        for disc in self.__discs:
+        for disc in self.__album.discs:
             for widget in [
                 self._tracks_widget_left[disc.number],
                 self._tracks_widget_right[disc.number]
@@ -176,7 +143,7 @@ class AlbumTracksView(TracksView):
             Get widget discs
             @return [Discs]
         """
-        return self.__discs
+        return self.__album.discs
 
     @property
     def is_populated(self):
@@ -195,12 +162,11 @@ class AlbumTracksView(TracksView):
         """
         if TracksView._init(self):
             if self.view_type & ViewType.SINGLE_COLUMN:
-                self.__discs = [self.__album.one_disc]
-            else:
-                self.__discs = self.__album.discs
-            for disc in self.__discs:
+                self.__album.merge_discs()
+            for disc in self.__album.discs:
                 self._add_disc_container(disc.number)
-            self.__discs_to_load = list(self.__discs)
+            self.__discs_to_load = list(self.__album.discs)
+            self.__show_disc_header = len(self.__discs_to_load) > 1
 
     def _add_tracks(self, widget, tracks, position=0):
         """
@@ -212,7 +178,7 @@ class AlbumTracksView(TracksView):
             # If user does not want to show real tracknumber and we are
             # in album view, calculate a fake tracknumber
             if not self.__show_tag_tracknumber and\
-                    self.view_type & ViewType.ALBUM:
+                    self.view_type & (ViewType.ALBUM | ViewType.ARTIST):
                 track.set_number(position + 1)
             row = TrackRow(track, self.__album.artist_ids, self.view_type)
             row.show()
@@ -239,35 +205,15 @@ class AlbumTracksView(TracksView):
         # ---------Label--------- #
         # | Column 1 | Column 2 | #
         ###########################
-        for disc in self.__discs:
-            show_label = len(self.__discs) > 1
-            disc_names = self.__album.disc_names(disc.number)
-            if show_label or disc_names:
-                if disc_names:
-                    disc_text = ", ".join(disc_names)
-                elif show_label:
-                    disc_text = _("Disc %s") % disc.number
-                label = Gtk.Label.new()
-                label.set_ellipsize(Pango.EllipsizeMode.END)
-                label.set_text(disc_text)
-                label.set_property("halign", Gtk.Align.START)
-                label.get_style_context().add_class("dim-label")
-                label.show()
-                eventbox = Gtk.EventBox()
-                eventbox.connect("realize", set_cursor_type)
-                eventbox.set_tooltip_text(_("Play"))
-                eventbox.connect("button-press-event",
-                                 self.__on_disc_button_press_event,
-                                 disc)
-                eventbox.add(label)
-                eventbox.show()
-                if orientation == Gtk.Orientation.VERTICAL:
-                    self._responsive_widget.attach(
-                        eventbox, 0, idx, 1, 1)
-                else:
-                    self._responsive_widget.attach(
-                        eventbox, 0, idx, 2, 1)
-                idx += 1
+        for disc in self.__album.discs:
+            label = self.__get_disc_label(disc.number)
+            if orientation == Gtk.Orientation.VERTICAL:
+                self._responsive_widget.attach(
+                    label, 0, idx, 1, 1)
+            else:
+                self._responsive_widget.attach(
+                    label, 0, idx, 2, 1)
+            idx += 1
             if orientation == Gtk.Orientation.VERTICAL:
                 self._responsive_widget.attach(
                           self._tracks_widget_left[disc.number],
@@ -295,39 +241,12 @@ class AlbumTracksView(TracksView):
             @param status as bool
             @param track as Track
         """
+        for child in self.children:
+            child.get_parent().unselect_row(child)
+        self._disallow_selection()
         if self.__album.is_web:
             TracksView._on_loading_changed(self, player, status, track)
 
-    def _on_album_updated(self, scanner, album_id):
-        """
-            On album modified, disable it
-            @param scanner as CollectionScanner
-            @param album_id as int
-        """
-        if self.__album.id != album_id:
-            return
-        removed = False
-        for dic in [self._tracks_widget_left, self._tracks_widget_right]:
-            for widget in dic.values():
-                for child in widget.get_children():
-                    if child.track.album.id == Type.NONE:
-                        removed = True
-        if removed:
-            for dic in [self._tracks_widget_left, self._tracks_widget_right]:
-                for widget in dic.values():
-                    for child in widget.get_children():
-                        child.destroy()
-            self.__discs = list(self.__discs)
-            self.__set_duration()
-            self.populate()
-
-    def __on_track_row_removed(self, row):
-        """
-            Pass signal
-            @param row as TrackRow
-        """
-        emit_signal(self, "track-removed", row)
-
     def _on_activated(self, widget, track):
         """
             Handle playback if album or pass signal
@@ -337,7 +256,7 @@ class AlbumTracksView(TracksView):
         if self.view_type & (ViewType.ALBUM | ViewType.ARTIST):
             tracks = []
             for child in self.children:
-                if child.track.loved != -1:
+                if child.track.loved != -1 or track.id == child.track.id:
                     tracks.append(child.track)
                 child.set_state_flags(Gtk.StateFlags.NORMAL, True)
             # Do not update album list if in party or album already available
@@ -356,9 +275,67 @@ class AlbumTracksView(TracksView):
         else:
             emit_signal(self, "activated", track)
 
+    def _on_row_selected(self, listbox, row):
+        """
+            Update album tracks
+            @param listbox as Gtk.ListBox
+            @param row as Gtk.ListBoxRow
+        """
+        TracksView._on_row_selected(self, listbox, row)
+        selected_rows = []
+        for child in self.children:
+            if child.is_selected():
+                selected_rows.append(child)
+        if selected_rows:
+            tracks = [row.track for row in selected_rows]
+            self.__album.set_tracks(tracks)
+        else:
+            self.__album.reset_tracks()
+
 #######################
 # PRIVATE             #
 #######################
+    def __get_disc_label(self, disc_number):
+        """
+            Get disc label widget
+            @param disc_number as int
+            @return Gtk.Label
+        """
+        if disc_number in self.__disc_labels.keys():
+            label = self.__disc_labels[disc_number]
+        else:
+            label = LabelWidget()
+            label.set_ellipsize(Pango.EllipsizeMode.END)
+            label.set_property("halign", Gtk.Align.START)
+            label.get_style_context().add_class("dim-label")
+            label.connect("clicked", self.__on_label_clicked, disc_number)
+            label.set_tooltip_text(_("Play"))
+            self.__disc_labels[disc_number] = label
+        return label
+
+    def __set_disc_header(self, disc_number):
+        """
+            Set disc header
+            @param disc_number as int
+        """
+        if self.__show_disc_header:
+            label = self.__get_disc_label(disc_number)
+            disc_names = self.__album.disc_names(disc_number)
+            if disc_names:
+                label_str = ", ".join(disc_names)
+            else:
+                label_str = _("Disc %s") % disc_number
+            disc_year = App().tracks.get_year_for_album(
+                self.__album.id, disc_number)
+            if disc_year != self.__album.year:
+                markup = "%s  <span size='small' alpha='50000'>(%s)</span>"
+                markup = markup % (GLib.markup_escape_text(label_str),
+                                   disc_year)
+            else:
+                markup = GLib.markup_escape_text(label_str)
+            label.set_markup(markup)
+            label.show()
+
     def __load_disc(self, items, disc_number, position=0):
         """
             Load discs
@@ -375,13 +352,19 @@ class AlbumTracksView(TracksView):
         else:
             emit_signal(self, "populated")
 
-    def __on_disc_button_press_event(self, button, event, disc):
+    def __on_track_row_removed(self, row):
+        """
+            Pass signal
+            @param row as TrackRow
+        """
+        emit_signal(self, "track-removed", row)
+
+    def __on_label_clicked(self, label, disc_number):
         """
             Add disc to playback
-            @param button as Gtk.Button
-            @param event as Gdk.ButtonEvent
-            @param disc as Disc
+            @param label as LabelWidget
+            @param disc_number as int
         """
-        album = Album(disc.album.id)
-        album.set_tracks(disc.tracks)
+        album = Album(self.__album.id)
+        album.set_disc_number(disc_number)
         App().player.play_album(album)
