@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2020 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2021 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -75,7 +75,6 @@ class AlbumWidget(Gtk.Grid):
         self.__revealer.set_transition_type(
                 Gtk.RevealerTransitionType.NONE)
         self.__populate()
-        self.__revealer.set_reveal_child(True)
 
     def set_selection(self):
         """
@@ -86,8 +85,6 @@ class AlbumWidget(Gtk.Grid):
             self.set_state_flags(Gtk.StateFlags.SELECTED, False)
         else:
             self.unset_state_flags(Gtk.StateFlags.SELECTED)
-        if self.__tracks_view is not None:
-            self.__tracks_view.set_playing_indicator()
 
     def set_duration(self, track_id):
         """
@@ -148,6 +145,7 @@ class AlbumWidget(Gtk.Grid):
             Get filtered children
             @return [Gtk.Widget]
         """
+        filtered = []
         if self.__tracks_view is None:
             self.__populate()
         filtered = self.__tracks_view.children
@@ -180,17 +178,34 @@ class AlbumWidget(Gtk.Grid):
         """
             Populate the view with album
         """
-        if self.__tracks_view is None:
-            self.__tracks_view = AlbumTracksView(self.__album,
-                                                 self.__view_type)
-            self.__tracks_view.show()
-            self.__tracks_view.set_margin_start(MARGIN)
-            self.__tracks_view.set_margin_end(MARGIN)
-            self.__tracks_view.populate()
-            self.__tracks_view.connect("populated", self.__on_tracks_populated)
-            self.__revealer.add(self.__tracks_view)
-        self.__revealer.set_reveal_child(
-            not self.__revealer.get_reveal_child())
+        def init_tracks_view():
+            if self.__tracks_view is None:
+                self.__tracks_view = AlbumTracksView(self.__album,
+                                                     self.__view_type)
+                self.__tracks_view.show()
+                self.__tracks_view.set_margin_start(MARGIN)
+                self.__tracks_view.set_margin_end(MARGIN)
+                self.__tracks_view.populate()
+                self.__tracks_view.connect("populated",
+                                           self.__on_tracks_populated)
+                self.__tracks_view.connect("selected",
+                                           self.__on_track_selected)
+                self.__revealer.add(self.__tracks_view)
+
+        def reveal(ignore, update_duration):
+            if update_duration:
+                self.get_style_context().remove_class("load-animation")
+                self.__banner.update_duration()
+            self.__revealer.set_reveal_child(
+                not self.__revealer.get_reveal_child())
+        init_tracks_view()
+        if self.__album.tracks:
+            reveal(None, False)
+        else:
+            self.get_style_context().add_class("load-animation")
+            App().task_helper.run(self.__album.load_tracks,
+                                  None,
+                                  callback=(reveal, True))
 
     def __on_tracks_populated(self, view):
         """
@@ -203,6 +218,14 @@ class AlbumWidget(Gtk.Grid):
             self.__revealer.set_transition_type(
                 Gtk.RevealerTransitionType.SLIDE_DOWN)
 
+    def __on_track_selected(self, view, selected):
+        """
+            Mark banner as filtered
+            @param view as TracksView
+            @param selected as bool
+        """
+        self.__banner.set_selected(selected)
+
     def __on_banner_populated(self, widget):
         """
             Emit populated signal
