@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2020 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2021 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -56,12 +56,12 @@ class RoundedAlbumsWidget(RoundedFlowBoxWidget):
             Set artwork
         """
         RoundedFlowBoxWidget.set_artwork(self)
-        if App().art.artwork_exists_in_cache(self.artwork_name,
-                                             "ROUNDED",
-                                             self._art_size,
-                                             self._art_size):
+        if App().art.exists_in_cache(self.artwork_name,
+                                     "ROUNDED",
+                                     self._art_size,
+                                     self._art_size):
             App().task_helper.run(
-                App().art.get_artwork_from_cache,
+                App().art.get_from_cache,
                 self.artwork_name, "ROUNDED",
                 self._art_size, self._art_size,
                 callback=(self.__on_load_from_cache,))
@@ -91,10 +91,10 @@ class RoundedAlbumsWidget(RoundedFlowBoxWidget):
         album_scaled_pixbufs = []
         while album_ids and len(album_pixbufs) != 9:
             album_id = album_ids.pop(0)
-            pixbuf = App().art.get_album_artwork(Album(album_id),
-                                                 self._art_size,
-                                                 self._art_size,
-                                                 self._scale_factor)
+            pixbuf = App().album_art.get(Album(album_id),
+                                         self._art_size,
+                                         self._art_size,
+                                         self._scale_factor)
             if pixbuf is not None:
                 album_pixbufs.append(pixbuf)
         if len(album_pixbufs) == 0:
@@ -117,7 +117,6 @@ class RoundedAlbumsWidget(RoundedFlowBoxWidget):
             newpix = pixbuf.scale_simple(self.__cover_size,
                                          self.__cover_size,
                                          GdkPixbuf.InterpType.NEAREST)
-            del pixbuf
             album_scaled_pixbufs.append(newpix)
 
         if len(album_scaled_pixbufs) == 0:
@@ -157,12 +156,8 @@ class RoundedAlbumsWidget(RoundedFlowBoxWidget):
             return
         rounded = get_round_surface(
             surface, self._scale_factor, self._art_size / 4)
-        del surface
         self._artwork.set_from_surface(rounded)
-        App().art.add_artwork_to_cache(self.artwork_name,
-                                       rounded,
-                                       "ROUNDED")
-        del rounded
+        App().art.add_to_cache(self.artwork_name, rounded, "ROUNDED")
         emit_signal(self, "populated")
 
     def __draw_surface(self, surface, ctx, positions,
@@ -185,14 +180,12 @@ class RoundedAlbumsWidget(RoundedFlowBoxWidget):
             y *= self.__cover_size
             subsurface = Gdk.cairo_surface_create_from_pixbuf(
                 pixbuf, self._scale_factor, None)
-            del pixbuf
             ctx.translate(x, y)
             ctx.set_source_surface(subsurface, 0, 0)
             ctx.paint()
             ctx.translate(-x, -y)
             self.__draw_surface(surface, ctx, positions,
                                 album_pixbufs, set_surface)
-            del surface
         if self.__cancellable.is_cancelled():
             return
         elif album_pixbufs and len(positions) > 0:
