@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2020 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2021 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -38,7 +38,8 @@ class ArtworkPlayerWidget(Gtk.Image, SignalsHelper):
             "allow-per-track-cover")
         return [
             (App().player, "current-changed", "_on_current_changed"),
-            (App().art, "album-artwork-changed", "_on_album_artwork_changed"),
+            (App().album_art, "album-artwork-changed",
+             "_on_album_artwork_changed"),
         ]
 
     def set_art_size(self, width, height):
@@ -88,9 +89,9 @@ class ArtworkPlayerWidget(Gtk.Image, SignalsHelper):
             if self.__per_track_cover:
                 behaviour |= ArtBehaviour.NO_CACHE
                 album = Album(App().player.current_track.album.id)
-                App().art.clean_album_cache(album,
-                                            width * scale_factor,
-                                            height * scale_factor)
+                App().album_art.clean(album,
+                                      width * scale_factor,
+                                      height * scale_factor)
                 album.set_tracks([App().player.current_track])
             else:
                 album = App().player.current_track.album
@@ -132,6 +133,7 @@ class ArtworkPlayerWidget(Gtk.Image, SignalsHelper):
             Set artwork
             @param surface as str
         """
+        self.set_size_request(self.__width, self.__height)
         context_style = self.get_style_context()
         if surface is None:
             context_style.add_class("cover-background")
@@ -140,10 +142,8 @@ class ArtworkPlayerWidget(Gtk.Image, SignalsHelper):
             icon_name = "folder-music-symbolic"
             self.set_from_icon_name(icon_name, Gtk.IconSize.INVALID)
             self.set_pixel_size(self.__width / 3)
-            self.set_size_request(self.__width, self.__height)
         else:
             context_style.remove_class("cover-background")
             if self.__behaviour & ArtBehaviour.ROUNDED:
                 context_style.remove_class("rounded")
             self.set_from_surface(surface)
-            del surface
